      INCLUDE 'H_FUN_v2.F90'
!
      PROGRAM HFUNIV_NO_DECIMALS
!
!     Program evaluates the accuracy of a given algorithm providing the
!     H-function values by comparison with the reference set containing
!     these values calculated with accuracy of 21 digits. In the
!     directory REFERENSE_SET, the file REF_HFUN_VALUES is present with
!     data calculated for both arguments, directional variable and
!     albedo, distribuled in linearly in the range 0<=mu<=1 in steps
!     0.01 (in total 10000 values). 
!
!     Three files are created by this program:
!     NDECIMALS_16 - Contains pairs of arguments for which the accuracy
!                    of 16 decimals was identified. If the reference
!                    value is equal to the calculated value within double
!                    precision arithmetic, the number of decimals is 
!                    arbitraliry assumed to be 17.
!     NDECIMALS_15 - Contains pairs of arguments for which the accuracy
!                    of 15 decimals was identified.
!     NDECIM_LT_15 - Contains pairs of arguments for which the accuracy 
!                    lower than 15 decimals was found (this file is
!                    in the case of presently analyzed subroutine HFDE).
!
!     A. Jablonski
!     Institute of Physical Chemistry, Polish Academy of Sciences
!     Kasprzaka 44/52, 01-224 Warsaw, Poland
!     e-mail: ajablonski@ichf.edu pl
!     23 January 2020
!     
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
	  CHARACTER CHN*10
!	  
      CHN='NDECIMALS_'
      KMAX=100
      JMAX=100
      OPEN(1,FILE='./REFERENCE_SET/REF_HFUN_VALUES.TXT',STATUS='OLD')                   	  
!
      OPEN(14,FILE=CHN//'16.TXT',STATUS='UNKNOWN')
      OPEN(15,FILE=CHN//'15.TXT',STATUS='UNKNOWN')
      OPEN(18,FILE='NDECIM_LT_15.TXT',STATUS='UNKNOWN')
!      
      WRITE(14,*)'Directional  Albedo  Decimals'
      WRITE(14,*)'variable mu  omega'      
      WRITE(14,*)' '
      WRITE(15,*)'Directional  Albedo  Decimals'
      WRITE(15,*)'variable mu  omega'      
      WRITE(15,*)' '
      WRITE(18,*)'Directional  Albedo  Decimals'
      WRITE(18,*)'variable mu  omega'      
      WRITE(18,*)' '                 
!
      DO 31 K=1,KMAX
      DO 30 J=1,JMAX 
!
      READ(1,*) X,OMEGA,HA 
      CALL HFUNIV(X,OMEGA,HB,INDEX)
      IF (HA .EQ. HB) NDECIM=17
      IF (HA .NE. HB) NDECIM=-IDNINT(DLOG10(DABS((HA-HB)/HA)))
      IF (NDECIM .GE. 16) WRITE(14,101)X,OMEGA,NDECIM
      IF (NDECIM .EQ. 15) WRITE(15,101)X,OMEGA,NDECIM
      IF (NDECIM .LT. 15) WRITE(18,101)X,OMEGA,NDECIM
      WRITE(*,101)X,OMEGA,NDECIM  
! 
  30  CONTINUE 
  31  CONTINUE
 100  FORMAT(1X,2I4,1X,2(ES13.5,1X),2X,I3)
 101  FORMAT(4X,F5.2,5X,F5.2,5X,I3)
!   
      CLOSE(18)
!      CLOSE(17)
!      CLOSE(16)
      CLOSE(15)
      CLOSE(14)
      CLOSE(1) 
!
      STOP
      END
