      INCLUDE 'H_FUN_v2.F90'
!
      PROGRAM TEST_PROGRAM_v2
!
!     ##################################################################
!
!     THIS PROGRAM IS TESTING ACCURACY OF PARAMETERS CALCULATED FROM 
!     INTEGRATION OF EXPRESSIONS CONTAINING THE CHANDRASEKHAR FUNCTION.
!     SELECTED APPLICATIONS OF THE CHANDRASEKHAR FUNCTION IN THE 
!     FORMALISM OF ELECTRON TRANSPORT IN ELEMENTAL SOLIDS IS
!     DEMONSTRATED:
!     (1) CALCULATIONS OF THE PHOTOELECTRON ESCAPE DEPTH
!     (2) CALCULATIONS OF THE PHOTOELECTRON SIGNAL INTENSITY 
!
!     A. Jablonski
!     Institute of Physical Chemistry, Polish Academy of Sciences
!     Kasprzaka 44/52, 01-224 Warsaw, Poland
!     e-mail: ajablonski@ichf.edu pl
!     23 January 2020
!
!     ##################################################################
!
      IMPLICIT DOUBLE PRECISION (A-H,O-Z),INTEGER*4 (I-N)
      COMMON/AAA/XMU,OMEGA,N
      CHARACTER CHZ*2,CHE*4,CHRAD*10,CHS*5,CHL*5      
      COMMON/EEE/CHL(30)
      COMMON /DDD/LINL(30),SIGL(30),BEL(30),BETAL(30),EKL(30)
      DIMENSION MAT(10)            
      PARAMETER (PI=3.14159265358979323846264338328D0,CONV=180.0D0/PI)
      PARAMETER (AVO=6.02214129D23)  ! AVOGADRO CONSTANT [1/MOLE]
      EXTERNAL FDELTA,FMOM      
!
      WRITE(*,*)'SELECT ONE OF THE FOLLOWING OPTIONS '
      WRITE(*,*)'ENTER FLAG : '
      WRITE(*,*)' '
      WRITE(*,*)'  1 - TEST OF ACCURACY OF THE CHANDRASEKHAR FUNCTION'
      WRITE(*,*)'  2 - APPLICATIONS: PHOTOELECTRON MEAN ESCAPE DEPTH'
      WRITE(*,*)'  3 - APPLICATIONS: PHOTOELECTRON YIELD'      
      READ(*,*)IOPTION
      IF (IOPTION .LT. 1 .OR. IOPTION .GT. 3) THEN
        WRITE(*,*)'SELECTED OPTION IS OUT OF THE ALLOWED RANGE'
        STOP
      ENDIF	  	  		            
!
!     ##################################################################
!
!     TEST OF ACCURACY OF THE CHANDRASEKHAR FUNCTION. CALCULATIONS OF
!     PARAMETERS ALPHA AND DELTA
!
      IF (IOPTION .EQ. 1) THEN
!
!     ##################################################################
!      
      WRITE(*,*)'ELECTRON EMISSION ANGLE WITH RESPECT TO SURFACE NORMAL'
      WRITE(*,*)'(0 DEG <= ALPHA <= 85 DEG) '
      READ(*,*)ALPHA
      IF (ALPHA .LT. 0.0D0 .OR. ALPHA .GT. 85.0D0) THEN
        WRITE(*,*)'SELECTED EMISSION ANGLE IS OUT OF THE RANGE '
        WRITE(*,*)'0 DEG <= Z <= 85 DEG '
        STOP
      ENDIF
      XMU=DCOS(ALPHA/CONV)    
!
!     ******************************************************************
! 
      E0=1.0D-13
      A=0.0D0
      B=1.0D0      
      OPEN(2,FILE='REPORT OF TESTS.TXT',STATUS='UNKNOWN')
      WRITE(2,400)ALPHA
      WRITE(2,401)XMU    
!     
      WRITE(2,*)' '              
      WRITE(2,102)
      WRITE(2,*)' '
      OMEGA=5.0D-4
      IH=2
      DO 30 I=1,4
	  N=0
      CALL GL64(FDELTA,A,B,E0,DELTA0,R,I1)
      CALL DANALYT(Y0,Y1)            
      IF (DELTA0 .EQ. Y0)IDELTA=18
      IF (DELTA0 .NE. Y0)IDELTA=-IDNINT(DLOG10(DABS((DELTA0-Y0)/Y0)))
      WRITE(*,300)OMEGA,DELTA0,Y0,IDELTA
      WRITE(2,300)OMEGA,DELTA0,Y0,IDELTA
      OMEGA=OMEGA*IH
      IF (IH .EQ. 2) THEN
        IH=5
      ELSE
        IH=2
      ENDIF 
  30  CONTINUE
!
      OMEGA=0.05D0
      DO 40 I=1,17
      IF (I .EQ. 17) OMEGA=0.85D0 
      N=0
      CALL GL64(FDELTA,A,B,E0,DELTA0,R,I1)
      CALL DANALYT(Y0,Y1)      
      IF (DELTA0 .EQ. Y0)IDELTA=18
      IF (DELTA0 .NE. Y0)IDELTA=-IDNINT(DLOG10(DABS((DELTA0-Y0)/Y0)))
      WRITE(*,300)OMEGA,DELTA0,Y0,IDELTA
      WRITE(2,300)OMEGA,DELTA0,Y0,IDELTA 
      OMEGA=OMEGA+0.05D0      
  40  CONTINUE
!
!     ******************************************************************
! 
      WRITE(2,*)' ' 
      WRITE(2,*)' '              
      WRITE(2,103)
      WRITE(2,*)' '
      OMEGA=5.0D-4
      IH=2
      DO 50 I=1,4
	  N=1
      CALL GL64(FDELTA,A,B,E0,DELTA1,R,I1)
      CALL DANALYT(Y0,Y1)            
      IF (DELTA1 .EQ. Y1)IDELTA=18
      IF (DELTA1 .NE. Y1)IDELTA=-IDNINT(DLOG10(DABS((DELTA1-Y1)/Y1)))
      WRITE(*,300)OMEGA,DELTA1,Y1,IDELTA
      WRITE(2,300)OMEGA,DELTA1,Y1,IDELTA
      OMEGA=OMEGA*IH
      IF (IH .EQ. 2) THEN
        IH=5
      ELSE
        IH=2
      ENDIF 
  50  CONTINUE
!
      OMEGA=0.05D0
      DO 60 I=1,17
      IF (I .EQ. 17) OMEGA=0.85D0 
      N=1
      CALL GL64(FDELTA,A,B,E0,DELTA1,R,I1)
      CALL DANALYT(Y0,Y1)      
      IF (DELTA1 .EQ. Y1)IDELTA=18
      IF (DELTA1 .NE. Y1)IDELTA=-IDNINT(DLOG10(DABS((DELTA1-Y1)/Y1)))
      WRITE(*,300)OMEGA,DELTA1,Y1,IDELTA
      WRITE(2,300)OMEGA,DELTA1,Y1,IDELTA 
      OMEGA=OMEGA+0.05D0      
  60  CONTINUE
!
!     ******************************************************************
! 
      WRITE(2,*)' ' 
      WRITE(2,*)' '   
      WRITE(2,104)
      WRITE(2,*)' '
      OMEGA=5.0D-4
      IH=2
      DO 70 I=1,4
	  N=0
      CALL GL64(FMOM,A,B,E0,ALPHA0,R,I0)
      Y0=2.0D0*(1.0D0-DSQRT(1.0D0-OMEGA))/OMEGA      
      IF (ALPHA0 .EQ. Y0)IDELTA=18
      IF (ALPHA0 .NE. Y0)IDELTA=-IDNINT(DLOG10(DABS((ALPHA0-Y0)/Y0)))
      WRITE(*,300)OMEGA,ALPHA0,Y0,IDELTA
      WRITE(2,300)OMEGA,ALPHA0,Y0,IDELTA
      OMEGA=OMEGA*IH
      IF (IH .EQ. 2) THEN
        IH=5
      ELSE
        IH=2
      ENDIF 
  70  CONTINUE 
!
      OMEGA=0.05D0
      DO 80 I=1,17
      IF (I .EQ. 17) OMEGA=0.85D0 
      N=0
      CALL GL64(FMOM,A,B,E0,ALPHA0,R,I0)
      Y0=2.0D0*(1.0D0-DSQRT(1.0D0-OMEGA))/OMEGA 
      IF (ALPHA0 .EQ. Y0)IDELTA=18
      IF (ALPHA0 .NE. Y0)IDELTA=-IDNINT(DLOG10(DABS((ALPHA0-Y0)/Y0)))
      WRITE(*,300)OMEGA,ALPHA0,Y0,IDELTA
      WRITE(2,300)OMEGA,ALPHA0,Y0,IDELTA 
      OMEGA=OMEGA+0.05D0      
  80  CONTINUE 
!
!     ******************************************************************
! 
      WRITE(2,*)' ' 
      WRITE(2,*)' '   
      WRITE(2,105)
      WRITE(2,*)' '
      OMEGA=5.0D-4
      IH=2
      DO 90 I=1,4
	  N=1
      CALL GL64(FMOM,A,B,E0,ALPHA1,R,I1)
      N=2
      CALL GL64(FMOM,A,B,E0,ALPHA2,R,I2)
      ALPHA12=DSQRT(1.0D0-OMEGA)*ALPHA2+OMEGA*ALPHA1*ALPHA1/4.0D0            
      Y12=1.0D0/3.0D0    
      IF (ALPHA12 .EQ. Y12)IDELTA=18
      IF (ALPHA12 .NE. Y12)                                             &
	 &IDELTA=-IDNINT(DLOG10(DABS((ALPHA12-Y12)/Y12)))
      WRITE(*,300)OMEGA,ALPHA12,Y12,IDELTA
      WRITE(2,300)OMEGA,ALPHA12,Y12,IDELTA
      OMEGA=OMEGA*IH
      IF (IH .EQ. 2) THEN
        IH=5
      ELSE
        IH=2
      ENDIF 
  90  CONTINUE 
!
      OMEGA=0.05D0
      DO 99 I=1,17
      IF (I .EQ. 17) OMEGA=0.85D0 
	  N=1
      CALL GL64(FMOM,A,B,E0,ALPHA1,R,I1)
      N=2
      CALL GL64(FMOM,A,B,E0,ALPHA2,R,I2)
      ALPHA12=DSQRT(1.0D0-OMEGA)*ALPHA2+OMEGA*ALPHA1*ALPHA1/4.0D0 
      Y12=1.0D0/3.0D0          
      IF (ALPHA12 .EQ. Y12)IDELTA=18
      IF (ALPHA12 .NE. Y12)                                             &
	 &IDELTA=-IDNINT(DLOG10(DABS((ALPHA12-Y12)/Y12)))
      WRITE(*,300)OMEGA,ALPHA12,Y12,IDELTA
      WRITE(2,300)OMEGA,ALPHA12,Y12,IDELTA
      OMEGA=OMEGA+0.05D0      
  99  CONTINUE 
!  
      CLOSE(2)
      STOP      
!
!     ##################################################################      
      ENDIF      
!     ##################################################################
!     ##################################################################
!
!     EXAMPLES OF APPLICATIONS OF THE H-FUNCTION. CALCULATIONS OF  
!     PHOTOELECTION MEAN ESCAPE DEPTH FOR SELECTED MEASUREMENTS
!     CONDITIONS
! 
      IF (IOPTION .EQ. 2 .OR. IOPTION .EQ. 3) THEN     
!
!     ##################################################################
!
      WRITE(*,*)'ATOMIC NUMBER (1 <= Z <=92) '
      READ(*,*)IZ
      IF (IZ .LT. 1 .OR. IZ .GT. 92) THEN
        WRITE(*,*)'SELECTED ATOMIC NUMBER IS OUT OF THE RANGE '
        WRITE(*,*)'1 <= Z <=92 '
        STOP
      ENDIF
!
!     ******************************************************************
!
      IA=1
      IF ((IZ .EQ. 6) .OR. (IZ .EQ. 15) .OR. (IZ .EQ. 34))              &
     &WRITE(*,*)'DEFINE THE ALLOTROPE'
      IF (IZ .EQ. 6) THEN
        WRITE(*,*)'ENTER FLAG (FROM 1 TO 3): '
        WRITE(*,*)'  1 - GRAPHITE '
        WRITE(*,*)'  2 - DIAMOND '
        WRITE(*,*)'  3 - GLASSY CARBON '
        READ(*,*)IA
        IF (IA .LT. 1 .OR. IA .GT. 3) THEN
          WRITE(*,*)'SELECTED OPTION IS OUT OF THE RANGE 1<= FLAG <=3'
          STOP
        ENDIF	  	
      ENDIF
      IF (IZ .EQ. 15) THEN
        WRITE(*,*)'ENTER FLAG (FROM 1 TO 3): '
        WRITE(*,*)'  1 - WHITE PHOSPHORUS '
        WRITE(*,*)'  2 - RED PHOSPHORUS '
        WRITE(*,*)'  3 - BLACK PHOSPHORUS '
        READ(*,*)IA
        IF (IA .LT. 1 .OR. IA .GT. 3) THEN
          WRITE(*,*)'SELECTED OPTION IS OUT OF THE RANGE 1<= FLAG <=3'
          STOP
        ENDIF	  	
      ENDIF
      IF (IZ .EQ. 34) THEN
        WRITE(*,*)'ENTER FLAG (FROM 1 TO 3): '
        WRITE(*,*)'  1 - ALPHA SELENIUM '
        WRITE(*,*)'  2 - VITREOUS SELENIUM '
        WRITE(*,*)'  3 - GRAY SELENIUM '
        READ(*,*)IA
        IF (IA .LT. 1 .OR. IA .GT. 3) THEN
          WRITE(*,*)'SELECTED OPTION IS OUT OF THE RANGE 1<= FLAG <=3'
          STOP
        ENDIF	  	
      ENDIF
!
!     ******************************************************************
!      
      WRITE(*,*)'ENTER FLAG : '
      WRITE(*,*)' '
      WRITE(*,*)'  1 - Mg K_alpha RADIATION' 
      WRITE(*,*)'  2 - Al K_alpha RADIATION'
      READ(*,*)IRAD
      IF (IRAD .LT. 1 .OR. IRAD .GT. 2) THEN
        WRITE(*,*)'SELECTED OPTION IS OUT OF THE ALLOWED RANGE'
        STOP
      ENDIF
      IF (IRAD .EQ. 1) CHRAD='Mg K_alpha'
      IF (IRAD .EQ. 2) CHRAD='Al K_alpha'
!      
      CALL DATAREAD(IZ,IRAD,K)
      WRITE(*,*)' '
      WRITE(*,*)' '      
      WRITE(*,*)'ENTER FLAG : '
      WRITE(*,*)' '            
      DO 10 I=1,K
      WRITE(*,199)I,CHL(I),IDNINT(EKL(I))   
  10  CONTINUE
      READ(*,*)ILINE
      IF (ILINE .LT. 1 .OR. ILINE .GT. K) THEN
        WRITE(*,*)'SELECTED OPTION IS OUT OF THE ALLOWED RANGE'
        STOP
      ENDIF
      BETA=BETAL(ILINE)
      SIGMA=SIGL(ILINE)
      ENER=EKL(ILINE)
      CHS=CHL(ILINE) 
      CALL OMEGAELE(IZ,IA,ENER,OMEGA,IFLAG)
      IF (IFLAG .NE. 1) THEN
        WRITE(*,*)'DATA ARE NOT AVAILABLE FOR SELECTED ELEMENT'
        STOP
      ENDIF
!
!     ******************************************************************
!
      CALL IMFPELE(IZ,IA,ENER,XIMFP,JJ)
      IF (JJ .NE. 1) THEN
        CALL IMFPTPP(IZ,IA,ENER,XIMFP,KK)
        IF (KK .NE. 1) THEN
          WRITE(*,*)                                                    &
     &'INELASTIC MEAN FREE PATH IS NOT AVAILABLE FOR SELECTED ELEMENT'
          STOP
        ENDIF
      ENDIF       
!
!     ******************************************************************
!     "MAGIC ANGLE" CONFIGURATION
!
      PSI=55.0D0
!      
      IZZ=IZ
      CALL DIGIT(IZZ,2,MAT)
      CHZ=CHAR(48+MAT(1))//CHAR(48+MAT(2))
      IEE=IDNINT(ENER)
      CALL DIGIT(IEE,4,MAT)      
      CHE=CHAR(48+MAT(1))//CHAR(48+MAT(2))//CHAR(48+MAT(3))//           &
     &CHAR(48+MAT(4))
!
!     ##################################################################
!     
      IF (IOPTION .EQ. 2) THEN            
      OPEN(1,FILE='MED_'//CHZ//'_'//CHE//'.TXT',STATUS='UNKNOWN')
      RMED=1.0D0-0.736D0*OMEGA
      EDPRED=XIMFP*RMED/10.0D0      
      WRITE(1,100)      
      WRITE(1,403)IZ
      IF (IZ .EQ.  6 .AND. IA .EQ. 1)                                   &
     &WRITE(1,*)'Allotrope                 :   Graphite' 
      IF (IZ .EQ.  6 .AND. IA .EQ. 2)                                   &
     &WRITE(1,*)'Allotrope                 :   Diamond'
      IF (IZ .EQ.  6 .AND. IA .EQ. 3)                                   &
     &WRITE(1,*)'Allotrope                 :   Glassy carbon'
      IF (IZ .EQ. 15 .AND. IA .EQ. 1)                                   &
     &WRITE(1,*)'Allotrope                 :   White phosphorus'
      IF (IZ .EQ. 15 .AND. IA .EQ. 2)                                   &
     &WRITE(1,*)'Allotrope                 :   Red phosphorus'
      IF (IZ .EQ. 15 .AND. IA .EQ. 3)                                   &
     &WRITE(1,*)'Allotrope                 :   Black phosphorus'
      IF (IZ .EQ. 34 .AND. IA .EQ. 1)                                   &
     &WRITE(1,*)'Allotrope                 :   Alpha selenium'
      IF (IZ .EQ. 34 .AND. IA .EQ. 2)                                   &
     &WRITE(1,*)'Allotrope                 :   Vitreous selenium'
      IF (IZ .EQ. 34 .AND. IA .EQ. 3)                                   &
     &WRITE(1,*)'Allotrope                 :   Gray selenium'
      WRITE(1,412)CHRAD
      WRITE(1,411)CHS
      WRITE(1,404)IDNINT(ENER)      
      WRITE(1,405)XIMFP/10.0D0
      WRITE(1,406)BETA
      WRITE(1,402)OMEGA
      WRITE(1,408)EDPRED
      WRITE(1,409)RMED 
      WRITE(1,106)
      WRITE(1,107)
      WRITE(1,108)                       
      ALPHA=0.0D0
      DO 500 I=1,86
      THETA=PSI-ALPHA
      XMU=DCOS(ALPHA/CONV)
      CALL ESCDEPTH(XIMFP,OMEGA,BETA,ALPHA,THETA,PSI,ED,EDMOD)    
      WRITE(*,410)IDNINT(ALPHA),ED/10.0D0,ED/(XMU*XIMFP),               &
     &EDMOD/10.0D0,EDMOD/(XMU*XIMFP) 
      WRITE(1,410)IDNINT(ALPHA),ED/10.0D0,ED/(XMU*XIMFP),               &
     &EDMOD/10.0D0,EDMOD/(XMU*XIMFP)  
      ALPHA=ALPHA+1.0D0
 500  CONTINUE                 
      CLOSE(1)
      ENDIF      
!
!     ##################################################################
!     
      IF (IOPTION .EQ. 3) THEN            
      OPEN(1,FILE='Yx_'//CHZ//'_'//CHE//'.TXT',STATUS='UNKNOWN')
      RQA=1.0D0-0.147D0*OMEGA-0.164D0*OMEGA*OMEGA
      CALL ATMASS(IZ,ATM,K)      
      CALL DENS(IZ,IA,RO,K) 
      FACTOR=1.0D-31*SIGMA*AVO*RO/ATM
      CPSI=DCOS(PSI/CONV)
      SIGCOM=FACTOR*XIMFP*(1.0D0-(BETA/4.0D0)*(3.0D0*CPSI*CPSI-1.0D0))      
      WRITE(1,600)    
      WRITE(1,403)IZ
      IF (IZ .EQ.  6 .AND. IA .EQ. 1)                                   &
     &WRITE(1,*)'Allotrope                 :   Graphite' 
      IF (IZ .EQ.  6 .AND. IA .EQ. 2)                                   &
     &WRITE(1,*)'Allotrope                 :   Diamond'
      IF (IZ .EQ.  6 .AND. IA .EQ. 3)                                   &
     &WRITE(1,*)'Allotrope                 :   Glassy carbon'
      IF (IZ .EQ. 15 .AND. IA .EQ. 1)                                   &
     &WRITE(1,*)'Allotrope                 :   White phosphorus'
      IF (IZ .EQ. 15 .AND. IA .EQ. 2)                                   &
     &WRITE(1,*)'Allotrope                 :   Red phosphorus'
      IF (IZ .EQ. 15 .AND. IA .EQ. 3)                                   &
     &WRITE(1,*)'Allotrope                 :   Black phosphorus'
      IF (IZ .EQ. 34 .AND. IA .EQ. 1)                                   &
     &WRITE(1,*)'Allotrope                 :   Alpha selenium'
      IF (IZ .EQ. 34 .AND. IA .EQ. 2)                                   &
     &WRITE(1,*)'Allotrope                 :   Vitreous selenium'
      IF (IZ .EQ. 34 .AND. IA .EQ. 3)                                   &
     &WRITE(1,*)'Allotrope                 :   Gray selenium'
      WRITE(1,412)CHRAD
      WRITE(1,411)CHS
      WRITE(1,404)IDNINT(ENER)      
      WRITE(1,405)XIMFP/10.0D0
      WRITE(1,406)BETA
      WRITE(1,402)OMEGA
      WRITE(1,601)SIGCOM*RQA
      WRITE(1,602)RQA
      WRITE(1,106)
      WRITE(1,607)
      WRITE(1,608)        
      ALPHA=0.0D0
      DO 501 I=1,86
      THETA=PSI-ALPHA
      XMU=DCOS(ALPHA/CONV)
      CALL SIGINT(XIMFP,OMEGA,BETA,ALPHA,THETA,PSI,SIG,SIGMOD)                
      WRITE(*,410)IDNINT(ALPHA),SIG*FACTOR,SIG*FACTOR/SIGCOM,           &
     &SIGMOD*FACTOR,SIGMOD*FACTOR/SIGCOM 
      WRITE(1,410)IDNINT(ALPHA),SIG*FACTOR,SIG*FACTOR/SIGCOM,           &
     &SIGMOD*FACTOR,SIGMOD*FACTOR/SIGCOM
      ALPHA=ALPHA+1.0D0
 501  CONTINUE                 
      CLOSE(1)     
      ENDIF        
!      
!     ##################################################################
      ENDIF
!     ##################################################################
!
 102  FORMAT(2X,'OMEGA',9X,'DELTA_0',7X,'ACCURATE FORMULA',3X,          &
     &'DECIMALS')
 103  FORMAT(2X,'OMEGA',9X,'DELTA_1',7X,'ACCURATE FORMULA',3X,          &
     &'DECIMALS')
 104  FORMAT(2X,'OMEGA',9X,'ALPHA_0',7X,'ACCURATE FORMULA',3X,          &
     &'DECIMALS')     
 105  FORMAT(2X,'OMEGA',8X,'ALPHA_1_2',7X,'CONSTANT VALUE',3X,          &
     &'DECIMALS')
 100  FORMAT(12X,'MEAN ESCAPE DEPTH OF PHOTOELECTRONS (MED)',/)
 600  FORMAT(16X,'PHOTOELECTRON YIELD (ARB. UNITS)',/)
 106  FORMAT(1X,'EMISSION',3X,'ACCURATE FORMALISM',11X,                 &
     &'SIMPLIFIED FORMALISM')     
 107  FORMAT(2X,'ANGLE',4X,'MED',8X,'Ratio R_MED',8X,'MED',8X,          &
     &'Ratio R_MED')
 108  FORMAT(2X,'(deg)',4X,'(nm)',26X,'(nm)',/)
 199  FORMAT(2X,I2,' - ',A5,' LINE   (KINETIC ENERGY = ',I4,' eV)')           
 300  FORMAT(1X,1PE7.1,1X,0PF19.15,0PF19.15,4X,I3)  
 400  FORMAT(1X,'Emission angle, alpha     : ',F6.2,' degrees')
 412  FORMAT(1X,'X-ray radiation           :   ',A10)
 411  FORMAT(1X,'Photoelectron line        :   ',A5)
 401  FORMAT(1X,'Directional argument, mu  : ',F9.6)            
 403  FORMAT(1X,'Atomic number             : ',I4)
 404  FORMAT(1X,'Energy                    : ',I5,' eV')
 405  FORMAT(1X,'Inelastic mean free path  : ',F8.4,' nm')
 406  FORMAT(1X,'Asymmetry parameter       : ',F6.2)
 402  FORMAT(1X,'Albedo, omega             : ',F10.6)
 408  FORMAT(1X,'MED from predict. formula : ',F8.4,' nm')
 409  FORMAT(1X,'Predicted ratio R_MED     : ',F8.4,/)     
 410  FORMAT(3X,I3,4(2X,G13.6))
 601  FORMAT(1X,'Yield from pred. formula  :  ',G13.6,' arb. units')
 602  FORMAT(1X,'Predicted ratio R_YIELD   :  ',G13.6,/)
 607  FORMAT(2X,'ANGLE',5X,'YIELD',5X,'Ratio R_YIELD',7X,'YIELD',5X,    &
     &'Ratio R_YIELD')
 608  FORMAT(2X,'(deg)',3X,'(arb. u.)',21X,'(arb. u.)',/)         
! 
      STOP
      END
!
!     ##################################################################
!
      SUBROUTINE DATAREAD(IZ,IRAD,K)
!
!     THIS PROGRAM IS READING THE PHOTOELECTRON CHARACTERISTICS FOR
!     SELECTED ELEMENT AND RADIATION. RELEVANT PARAMETERS ARE TAKEN
!     FROM:
!
!     I.M. Band, Yu.I. Kharitonov, M.B. Trzhaskovskaya, Photoionization
!     cross sections and photoelectron angular distribution for X-ray
!     line energiers in the range 0.132-5.509 keV, At. Data Nucl. Data
!     Tables 23 (1979) 443-505.
!
!     INPUT PARAMETERS
!     IZ    - ATOMIC NUMBER
!     IRAD  - RADIATION
!             IRAD = 1  Mg K_alpha RADIATION
!             IRAD = 2  Al K_alpha RADIATION
!
!     OUTPUT PARAMETER
!     K     - NUMBER OF AVAILABLE PHOTOELECTRON LINES WITH BINDING
!             ENERGY EXCEEDING 30 eV
!
!     OUTPUT PARAMETWRS VIA COMMON /DDD/
!     LINL  - TABLE OF ENCODED PHOTOELECTRON LINE NOTATION
!     SIGL  - TABLE OF TOTAL PHOTOEMISSION CROSS SECTIONS (BARN)
!     BEL   - TABLE OF BINDING ENERGIES (eV)
!     BETAL - TABLE OF ASYMMETRY PARAMETERS
!     EKL   - TABLE OF PHOTOELECTRON KINETIC ENERGIES
!
!     OUTPUT PARAMETERS VIA COMMON /EEE/
!     CHL   - SPECTOSCOPIC NOTATION FOR AVAILABLE PHOTOELECTRON LINES
!                   
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      CHARACTER CHS,CHL*5
      COMMON/EEE/CHL(30)
      COMMON/DDD/LINL(30),SIGL(30),BEL(30),BETAL(30),EKL(30)      
      DIMENSION MAT(10)            
!    
      IF (IRAD .EQ. 1) THEN
        OPEN(1,FILE='BAND_MG.TXT',STATUS='OLD')
        MAXDATA=996
        LCORR=0
        ENRAD=1253.6D0
      ENDIF        
      IF (IRAD .EQ. 2) THEN
        OPEN(1,FILE='BAND_AL.TXT',STATUS='OLD')
        MAXDATA=1030
        LCORR=1
        ENRAD=1486.6D0
      ENDIF
!
      K=0
      DO 100 I=1,MAXDATA
      READ(1,*)IZTAB,LINTAB,SIGTAB,BETAB,BETATAB
      IF (IZTAB .GT. IZ) GOTO 101      
      IF (IZTAB .LT. IZ) GOTO 100
      IF (BETAB .LT. 30.0D0) GOTO 100
      IF (ENRAD-BETAB .LT. 50.0D0) GOTO 100      
      K=K+1
      LINL(K)=(LINTAB-LCORR)/10
      SIGL(K)=SIGTAB
      BEL(K)=BETAB
      BETAL(K)=BETATAB
      EKL(K)=ENRAD-BETAB      
 100  CONTINUE
 101  CONTINUE 
      CLOSE(1)
!
      DO 12 I=1,K
      ILL=LINL(I)  
      CALL DIGIT(ILL,3,MAT)
      IF (MAT(3) .EQ. 0) LL=2*MAT(2)-1
      IF (MAT(3) .EQ. 1) LL=2*MAT(2)+1
      IF (MAT(2) .EQ. 0) CHS='s'
      IF (MAT(2) .EQ. 1) CHS='p'
      IF (MAT(2) .EQ. 2) CHS='d'
      IF (MAT(2) .EQ. 3) CHS='f'
      CHL(I)=CHAR(48+MAT(1))//CHS//CHAR(48+LL)//'/2'
!      
  12  CONTINUE  
!     
      RETURN
      END
!
!     ******************************************************************
!
      SUBROUTINE DIGIT(N,L,M)
!
!     SUBPROGRAM DECOMPOSES AN INTEGER NUMBER INTO THE SEPARATE DIGITS.
!     LAST DIGIT OF THE INTEGER N IS CONTAINED IN L-TH ELEMENT OF THE
!     TABLE M. IF THE NUMBER N HAS L DIGITS, THE FIRST DIGIT IS
!     CONTAINED IN THE FIRST ELEMENT OF THE TABLE M. IF THE NUMBER N 
!     HAS LESS THAN L DIGITS, FIRST ELEMENTS OF THE TABLE M CONTAIN
!     ZEROS.
!
!     INPUT PARAMETERS
!     N - AN INTEGER NUMBER
!     L - NUMBER OF THE CONSIDERED DECIMAL PLACES
!
!     OUTPUT PARAMETERS
!     M - TABLE CONTAINING L DIGITS OF THE NUMBER N
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION M(10)
      DO 10 I=1,10
      M(I)=0
  10  CONTINUE
      DO 20 I=1,L
      J=MOD(N,10)
      N=(N-J)/10
      M(L-I+1)=J
  20  CONTINUE
      RETURN
      END 
!
!     ##################################################################
!      
      SUBROUTINE OMEGAELE(IZ,IA,ENER,OMEGA,IFLAG)
!
!     CALCULATIONS OF SINGLE SCATTERING ALBEDO, OMEGA, FOR FOR ELEMENTAL
!     SOLID IN THE ENERGY RANGE FROM 50 EV TO 30 KEV.
!     THE ALGORITHM AND THE PARAMETERS ARE TAKEN FROM:
!
!     A. Jablonski, J. Phys. D: Appl. Phys. 48 (2015) 075301 (2015);
!     Erratum: J. Phys. D: Appl. Phys. 49 (2016) 289501
!
!     INPUT PARAMETERS
!     IZ   - ATOMIC NUMBER
!     IA   - NUMBER OF ALLOTROPE
!     ENER - ENERGY (EV), 50 EV <= ENERGY <= 30000 EV
!
!     OUTPUT PARAMETERS
!     OMEGA - SINGLE SCATTERING ALBEDO
!     IFLAG - FLAG
!             IFLAG=-4 ENERGY VALUE OUT OF ACCEPTABLE RANGE
!             IFLAG=-3 ATOMIC NUMBER OUT OF ACCEPTABLE RANGE
!             IFLAG=-2 ALLOTROPE NUMBER OUT OF ACCEPTABLE RANGE
!             IFLAG=-1 ALLOTROPE NUMBER SHOULD BE EQUAL TO UNITY  
!             IFLAG= 0 DATA ARE NOT AVAILABLE FOR SELECTED ELEMENT
!      
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION IZE(78),IAE(78)      
      DIMENSION A1(78),A2(78),A3(78),A4(78),A5(78)
      DIMENSION B1(78),B2(78),B3(78),B4(78),B5(78)      
      DIMENSION AA(5),BB(5)
      DATA IZE / 3,  4,  5,  6,  6,  6, 11, 12, 13, 14, 15, 15, 15, 16, &
     &          19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, &
     &          33, 34, 34, 34, 37, 38, 39, 40, 41, 42, 44, 45, 46, 47, &
     &          48, 49, 50, 51, 52, 55, 56, 57, 58, 59, 60, 62, 63, 64, &
     &          65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, &
     &          79, 80, 81, 82, 83, 90, 91, 92/
!  
      DATA IAE / 1,  1,  1,  1,  2,  3,  1,  1,  1,  1,  1,  2,  3,  1, &
     &           1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, &
     &           1,  1,  2,  3,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, &
     &           1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, &
     &           1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, &
     &           1,  1,  1,  1,  1,  1,  1,  1/
!  
      DATA A1 /-1.043380D-2 ,-3.726850D-3 ,-3.089630D-2 ,-3.966010D-3 , &
     &         -1.147040D-2 ,-1.375680D-2 ,-2.993940D-2 ,-3.068490D-2 , &
     &         -1.360670D-2 ,-5.130600D-3 ,-1.228290D-2 ,-1.407730D-2 , &
     &         -1.634660D-2 ,-1.511130D-2 , 3.247710D-2 , 3.561870D-3 , &
     &          1.554800D-3 ,-1.260580D-2 ,-3.788080D-2 ,-1.209800D-3 , &
     &         -9.625250D-3 ,-3.346610D-2 ,-5.561700D-2 ,-5.829210D-2 , &
     &         -3.061270D-2 ,-1.425890D-2 , 4.267450D-2 , 3.635080D-2 , &
     &          9.721600D-2 , 1.472540D-1 , 1.481360D-1 , 1.442220D-1 , &
     &          1.602920D-1 , 8.240840D-2 , 5.376470D-2 ,-1.577850D-2 , &
     &         -9.595220D-2 ,-1.336950D-1 ,-2.681010D-1 ,-3.478680D-1 , &
     &         -4.615260D-1 ,-5.169410D-1 ,-5.240720D-1 ,-5.193580D-1 , &
     &         -5.070100D-1 ,-3.815190D-1 ,-3.069910D-1 , 4.712840D-2 , &
     &          8.101290D-2 , 1.139810D-1 , 3.136290D-2 ,-5.134670D-3 , &
     &         -5.981000D-2 ,-2.069090D-1 ,-2.811700D-1 ,-2.814350D-1 , &
     &         -4.494470D-1 ,-5.214310D-1 ,-5.598990D-1 ,-5.954020D-1 , &
     &         -6.145480D-1 ,-6.161020D-1 ,-5.739510D-1 ,-5.706290D-1 , &
     &         -5.443100D-1 ,-4.870290D-1 ,-4.470950D-1 ,-3.878860D-1 , &
     &         -3.303860D-1 ,-2.794310D-1 ,-2.442710D-1 ,-1.611440D-1 , &
     &         -6.166300D-2 ,-1.021760D-2 ,-1.052320D-2 , 1.729660D-1 , &
     &          2.397670D-1 , 2.754290D-1/
!  
      DATA A2 / 2.294890D-1 , 6.954230D-2 , 6.221360D-1 , 5.473960D-2 , &
     &          2.063360D-1 , 2.640140D-1 , 6.342910D-1 , 6.357150D-1 , &
     &          2.776660D-1 , 9.981150D-2 , 2.437500D-1 , 2.770470D-1 , &
     &          3.192080D-1 , 2.973920D-1 ,-6.615850D-1 ,-6.938070D-2 , &
     &         -5.475300D-2 , 2.372230D-1 , 7.480190D-1 , 2.497950D-3 , &
     &          1.741890D-1 , 6.614280D-1 , 1.079210D+0 , 1.132570D+0 , &
     &          5.600300D-1 , 2.499470D-1 ,-8.733850D-1 ,-7.232950D-1 , &
     &         -1.976600D+0 ,-2.991450D+0 ,-3.008500D+0 ,-2.932960D+0 , &
     &         -3.365440D+0 ,-1.831840D+0 ,-1.303430D+0 , 1.162910D-1 , &
     &          1.689380D+0 , 2.417330D+0 , 5.086850D+0 , 6.684570D+0 , &
     &          8.982740D+0 , 1.012580D+1 , 1.022200D+1 , 1.012710D+1 , &
     &          9.850230D+0 , 7.285000D+0 , 5.756130D+0 ,-1.270430D+0 , &
     &         -1.855520D+0 ,-2.489800D+0 ,-9.830890D-1 ,-3.148790D-1 , &
     &          7.149810D-1 , 3.545270D+0 , 4.979060D+0 , 5.001890D+0 , &
     &          8.310020D+0 , 9.771300D+0 , 1.057810D+1 , 1.133620D+1 , &
     &          1.178270D+1 , 1.188560D+1 , 1.111920D+1 , 1.116900D+1 , &
     &          1.070930D+1 , 9.637490D+0 , 8.913980D+0 , 7.827950D+0 , &
     &          6.750250D+0 , 5.826910D+0 , 5.205440D+0 , 3.555740D+0 , &
     &          1.556230D+0 , 5.333610D-1 , 5.800130D-1 ,-3.088980D+0 , &
     &         -4.441830D+0 ,-5.165970D+0/
!  
      DATA A3 /-1.943770D+0 ,-5.426750D-1 ,-4.752380D+0 ,-2.431470D-1 , &
     &         -1.388750D+0 ,-1.936930D+0 ,-5.146320D+0 ,-5.028860D+0 , &
     &         -2.208670D+0 ,-8.060460D-1 ,-1.885030D+0 ,-2.106850D+0 , &
     &         -2.388280D+0 ,-2.254290D+0 , 4.945510D+0 , 4.089960D-1 , &
     &          5.503970D-1 ,-1.713560D+0 ,-5.577660D+0 , 6.894740D-2 , &
     &         -1.244020D+0 ,-4.988470D+0 ,-7.896770D+0 ,-8.310990D+0 , &
     &         -3.911250D+0 ,-1.767120D+0 , 6.488620D+0 , 5.152830D+0 , &
     &          1.484240D+1 , 2.251680D+1 , 2.263690D+1 , 2.210570D+1 , &
     &          2.623540D+1 , 1.505260D+1 , 1.154170D+1 , 7.194170D-1 , &
     &         -1.074790D+1 ,-1.595110D+1 ,-3.569170D+1 ,-4.761280D+1 , &
     &         -6.491760D+1 ,-7.369020D+1 ,-7.404880D+1 ,-7.335610D+1 , &
     &         -7.104240D+1 ,-5.148070D+1 ,-3.978090D+1 , 1.202720D+1 , &
     &          1.564440D+1 , 2.017800D+1 , 1.012850D+1 , 5.634460D+0 , &
     &         -1.533680D+0 ,-2.171710D+1 ,-3.199090D+1 ,-3.227910D+1 , &
     &         -5.643600D+1 ,-6.744430D+1 ,-7.369000D+1 ,-7.964570D+1 , &
     &         -8.339760D+1 ,-8.465300D+1 ,-7.951190D+1 ,-8.082790D+1 , &
     &         -7.788770D+1 ,-7.044530D+1 ,-6.559690D+1 ,-5.826890D+1 , &
     &         -5.076280D+1 ,-4.457390D+1 ,-4.055840D+1 ,-2.833930D+1 , &
     &         -1.350410D+1 ,-5.952600D+0 ,-6.647550D+0 , 2.038280D+1 , &
     &          3.056000D+1 , 3.602180D+1/
!  
      DATA A4 / 6.630910D+0 , 1.337360D+0 , 1.567700D+1 ,-3.371610D-1 , &
     &          3.522840D+0 , 5.858290D+0 , 1.854420D+1 , 1.761710D+1 , &
     &          7.752270D+0 , 2.836410D+0 , 6.386070D+0 , 7.001680D+0 , &
     &          7.787570D+0 , 7.475430D+0 ,-1.626270D+1 ,-8.716330D-1 , &
     &         -2.456280D+0 , 5.410390D+0 , 1.840650D+1 ,-3.980850D-1 , &
     &          4.077340D+0 , 1.695080D+1 , 2.577130D+1 , 2.730510D+1 , &
     &          1.251600D+1 , 6.228680D+0 ,-2.049100D+1 ,-1.519790D+1 , &
     &         -4.855970D+1 ,-7.415610D+1 ,-7.451550D+1 ,-7.293150D+1 , &
     &         -8.973460D+1 ,-5.412320D+1 ,-4.443990D+1 ,-7.981060D+0 , &
     &          2.874000D+1 , 4.503970D+1 , 1.093900D+2 , 1.486290D+2 , &
     &          2.061180D+2 , 2.357400D+2 , 2.357580D+2 , 2.336150D+2 , &
     &          2.250520D+2 , 1.591890D+2 , 1.196470D+2 ,-4.811650D+1 , &
     &         -5.744130D+1 ,-7.172930D+1 ,-4.288170D+1 ,-2.975440D+1 , &
     &         -7.933070D+0 , 5.523110D+1 , 8.765810D+1 , 8.891550D+1 , &
     &          1.663480D+2 , 2.027740D+2 , 2.238950D+2 , 2.442920D+2 , &
     &          2.578250D+2 , 2.634290D+2 , 2.482770D+2 , 2.559180D+2 , &
     &          2.477520D+2 , 2.249850D+2 , 2.107100D+2 , 1.892080D+2 , &
     &          1.661600D+2 , 1.479850D+2 , 1.367760D+2 , 9.671730D+1 , &
     &          4.868390D+1 , 2.411890D+1 , 2.764220D+1 ,-5.915740D+1 , &
     &         -9.281510D+1 ,-1.109380D+2/  
!  
      DATA A5 /-8.971870D+0 ,-9.827360D-1 ,-1.905070D+1 , 2.195240D+0 , &
     &         -2.432430D+0 ,-6.360170D+0 ,-2.611080D+1 ,-2.393070D+1 , &
     &         -1.085340D+1 ,-4.385330D+0 ,-8.865140D+0 ,-9.329400D+0 , &
     &         -9.941090D+0 ,-9.915550D+0 , 1.875420D+1 ,-6.055330D-1 , &
     &          3.310740D+0 ,-6.919440D+0 ,-2.319780D+1 ,-4.639840D-2 , &
     &         -5.785850D+0 ,-2.254320D+1 ,-3.233940D+1 ,-3.466930D+1 , &
     &         -1.649660D+1 ,-1.031270D+1 , 2.171000D+1 , 1.368560D+1 , &
     &          5.690810D+1 , 8.841700D+1 , 8.877520D+1 , 8.720520D+1 , &
     &          1.118130D+2 , 7.043650D+1 , 6.175280D+1 , 1.625350D+1 , &
     &         -2.711050D+1 ,-4.595550D+1 ,-1.238720D+2 ,-1.719290D+2 , &
     &         -2.430470D+2 ,-2.800820D+2 ,-2.789100D+2 ,-2.765740D+2 , &
     &         -2.647850D+2 ,-1.822740D+2 ,-1.326220D+2 , 6.754350D+1 , &
     &          7.609770D+1 , 9.305080D+1 , 6.308130D+1 , 4.902220D+1 , &
     &          2.450690D+1 ,-4.868100D+1 ,-8.705980D+1 ,-8.864440D+1 , &
     &         -1.805800D+2 ,-2.251670D+2 ,-2.514190D+2 ,-2.771220D+2 , &
     &         -2.948770D+2 ,-3.036320D+2 ,-2.867080D+2 ,-2.999890D+2 , &
     &         -2.916140D+2 ,-2.655520D+2 ,-2.498900D+2 ,-2.267110D+2 , &
     &         -2.003730D+2 ,-1.806930D+2 ,-1.693640D+2 ,-1.204760D+2 , &
     &         -6.308150D+1 ,-3.337120D+1 ,-3.940060D+1 , 6.332580D+1 , &
     &          1.047170D+2 , 1.270950D+2/
!  
      DATA B1 /-1.365680D-4 ,-6.359920D-4 ,-8.427800D-4 ,-6.288120D-4 , &
     &         -6.211790D-4 ,-7.137620D-4 , 8.362240D-4 , 1.144640D-3 , &
     &          1.307650D-3 , 1.334980D-3 , 1.252980D-3 , 1.312940D-3 , &
     &          1.396670D-3 , 1.213930D-3 , 5.808300D-4 , 5.051840D-4 , &
     &          6.254460D-4 , 5.847180D-4 , 6.103720D-4 , 4.758430D-4 , &
     &          4.169110D-4 , 3.502720D-4 , 2.516640D-4 , 1.353390D-4 , &
     &         -1.157230D-4 ,-5.032670D-4 ,-5.404660D-4 ,-7.872130D-4 , &
     &         -6.570540D-4 ,-9.474510D-4 ,-9.723890D-4 ,-8.614700D-4 , &
     &         -1.029120D-3 ,-6.583640D-4 ,-3.324990D-4 , 6.569580D-5 , &
     &          2.620990D-4 , 2.200090D-4 , 1.982190D-4 , 1.138580D-4 , &
     &         -1.462580D-4 ,-1.019000D-4 ,-4.654120D-4 ,-3.990060D-4 , &
     &         -4.520250D-4 ,-5.334100D-4 ,-6.638050D-4 ,-1.319450D-3 , &
     &         -6.719870D-4 ,-3.533290D-4 ,-9.192570D-4 ,-1.170740D-3 , &
     &         -1.429490D-3 ,-2.004800D-3 ,-2.751170D-3 ,-2.463480D-3 , &
     &         -3.241460D-3 ,-3.502310D-3 ,-3.759030D-3 ,-4.142130D-3 , &
     &         -4.528940D-3 ,-5.354620D-3 ,-4.710670D-3 ,-3.927330D-3 , &
     &         -3.774660D-3 ,-3.486980D-3 ,-3.409640D-3 ,-3.168570D-3 , &
     &         -3.238870D-3 ,-3.621550D-3 ,-3.710000D-3 ,-3.784090D-3 , &
     &         -3.094620D-3 ,-2.654050D-3 ,-2.178720D-3 , 1.118520D-3 , &
     &          1.001710D-3 , 1.083190D-3/
!  
      DATA B2 / 6.255080D-3 , 2.459010D-2 , 3.385560D-2 , 2.762690D-2 , &
     &          2.797520D-2 , 3.104460D-2 ,-1.825060D-2 ,-2.899630D-2 , &
     &         -3.474470D-2 ,-3.632670D-2 ,-3.470500D-2 ,-3.664020D-2 , &
     &         -3.933460D-2 ,-3.403490D-2 ,-1.391590D-2 ,-1.176200D-2 , &
     &         -1.662340D-2 ,-1.527320D-2 ,-1.617210D-2 ,-1.147960D-2 , &
     &         -9.489370D-3 ,-7.252710D-3 ,-3.913970D-3 , 1.602600D-4 , &
     &          8.931920D-3 , 2.241820D-2 , 2.385010D-2 , 3.261300D-2 , &
     &          2.817390D-2 , 3.845250D-2 , 3.932310D-2 , 3.544910D-2 , &
     &          4.176360D-2 , 2.893980D-2 , 1.755280D-2 , 3.430610D-3 , &
     &         -3.712910D-3 ,-2.166940D-3 ,-1.495550D-3 , 1.465260D-3 , &
     &          1.048550D-2 , 9.133280D-3 , 2.203190D-2 , 1.943200D-2 , &
     &          2.163750D-2 , 2.431120D-2 , 2.906900D-2 , 5.230630D-2 , &
     &          2.905400D-2 , 1.745840D-2 , 3.794660D-2 , 4.671150D-2 , &
     &          5.572540D-2 , 7.578350D-2 , 1.023060D-1 , 9.172810D-2 , &
     &          1.191780D-1 , 1.282450D-1 , 1.371850D-1 , 1.506510D-1 , &
     &          1.642770D-1 , 1.938880D-1 , 1.712270D-1 , 1.427530D-1 , &
     &          1.377980D-1 , 1.275860D-1 , 1.252100D-1 , 1.163720D-1 , &
     &          1.193070D-1 , 1.327920D-1 , 1.365870D-1 , 1.413850D-1 , &
     &          1.148980D-1 , 1.010730D-1 , 8.703260D-2 ,-2.841920D-2 , &
     &         -2.631010D-2 ,-2.985430D-2/
!  
      DATA B3 /-9.925490D-2 ,-3.503340D-1 ,-5.025720D-1 ,-4.404420D-1 , &
     &         -4.545990D-1 ,-4.925850D-1 , 6.213770D-2 , 1.982680D-1 , &
     &          2.690980D-1 , 2.952630D-1 , 2.908180D-1 , 3.130290D-1 , &
     &          3.437280D-1 , 2.887300D-1 , 4.725880D-2 , 2.549630D-2 , &
     &          9.761140D-2 , 7.798070D-2 , 8.692710D-2 , 2.300110D-2 , &
     &         -2.522690D-3 ,-3.220390D-2 ,-7.332900D-2 ,-1.277900D-1 , &
     &         -2.439230D-1 ,-4.170070D-1 ,-4.393530D-1 ,-5.529850D-1 , &
     &         -4.970710D-1 ,-6.305770D-1 ,-6.417720D-1 ,-5.919480D-1 , &
     &         -6.819120D-1 ,-5.163750D-1 ,-3.680600D-1 ,-1.835680D-1 , &
     &         -8.763350D-2 ,-1.084470D-1 ,-1.160520D-1 ,-1.544840D-1 , &
     &         -2.711600D-1 ,-2.547180D-1 ,-4.253250D-1 ,-3.889480D-1 , &
     &         -4.203680D-1 ,-4.547420D-1 ,-5.183230D-1 ,-8.253360D-1 , &
     &         -5.151810D-1 ,-3.585770D-1 ,-6.327460D-1 ,-7.471330D-1 , &
     &         -8.647600D-1 ,-1.126770D+0 ,-1.478250D+0 ,-1.335320D+0 , &
     &         -1.696820D+0 ,-1.815100D+0 ,-1.931970D+0 ,-2.109350D+0 , &
     &         -2.289240D+0 ,-2.686000D+0 ,-2.389110D+0 ,-2.004260D+0 , &
     &         -1.944860D+0 ,-1.809900D+0 ,-1.783430D+0 ,-1.662970D+0 , &
     &         -1.707790D+0 ,-1.885460D+0 ,-1.944490D+0 ,-2.036550D+0 , &
     &         -1.659900D+0 ,-1.500650D+0 ,-1.351700D+0 , 1.582010D-1 , &
     &          1.577250D-1 , 2.138520D-1/
!  
      DATA B4 /-2.712040D-1 , 1.235800D+0 , 2.328160D+0 , 2.103180D+0 , &
     &          2.248870D+0 , 2.461950D+0 ,-7.204800D-3 ,-7.431300D-1 , &
     &         -1.085880D+0 ,-1.241090D+0 ,-1.307130D+0 ,-1.410560D+0 , &
     &         -1.550870D+0 ,-1.305930D+0 , 4.609970D-2 , 1.350950D-1 , &
     &         -3.223700D-1 ,-1.635950D-1 ,-1.692890D-1 , 2.490750D-1 , &
     &          4.029440D-1 , 6.023380D-1 , 8.208980D-1 , 1.161020D+0 , &
     &          1.864190D+0 , 2.835720D+0 , 3.013270D+0 , 3.644070D+0 , &
     &          3.345330D+0 , 4.095240D+0 , 4.157440D+0 , 3.880650D+0 , &
     &          4.482050D+0 , 3.539420D+0 , 2.693550D+0 , 1.655600D+0 , &
     &          1.105670D+0 , 1.229580D+0 , 1.277460D+0 , 1.498520D+0 , &
     &          2.170770D+0 , 2.067410D+0 , 3.065010D+0 , 2.865910D+0 , &
     &          3.039270D+0 , 3.258730D+0 , 3.624380D+0 , 5.434900D+0 , &
     &          3.622180D+0 , 2.701450D+0 , 4.290620D+0 , 4.958090D+0 , &
     &          5.644680D+0 , 7.174890D+0 , 9.231330D+0 , 8.410950D+0 , &
     &          1.051600D+1 , 1.120760D+1 , 1.189020D+1 , 1.293350D+1 , &
     &          1.399420D+1 , 1.634710D+1 , 1.464150D+1 , 1.237300D+1 , &
     &          1.206340D+1 , 1.128390D+1 , 1.116160D+1 , 1.044910D+1 , &
     &          1.074810D+1 , 1.178860D+1 , 1.218660D+1 , 1.287290D+1 , &
     &          1.056170D+1 , 9.758460D+0 , 9.083050D+0 , 3.782820D-1 , &
     &          2.424520D-1 ,-1.333190D-1/
!  
      DATA B5 / 9.153380D-1 ,-1.589400D+0 ,-3.817170D+0 ,-3.807740D+0 , &
     &         -3.944420D+0 ,-4.553180D+0 ,-1.231960D+0 , 3.938440D-1 , &
     &          1.134650D+0 , 1.462820D+0 , 1.574220D+0 , 1.847310D+0 , &
     &          2.196800D+0 , 1.662970D+0 ,-1.334200D+0 ,-1.325920D+0 , &
     &         -2.202760D-1 ,-5.166680D-1 ,-4.648580D-1 ,-1.431690D+0 , &
     &         -1.761170D+0 ,-2.244850D+0 ,-2.689930D+0 ,-3.478130D+0 , &
     &         -5.061200D+0 ,-7.217410D+0 ,-7.716680D+0 ,-9.115780D+0 , &
     &         -8.524820D+0 ,-1.023470D+1 ,-1.037370D+1 ,-9.756430D+0 , &
     &         -1.137460D+1 ,-9.318480D+0 ,-7.488350D+0 ,-5.156350D+0 , &
     &         -3.953390D+0 ,-4.243540D+0 ,-4.362740D+0 ,-4.853470D+0 , &
     &         -6.326770D+0 ,-6.093580D+0 ,-8.330080D+0 ,-7.932860D+0 , &
     &         -8.339000D+0 ,-8.823930D+0 ,-9.666360D+0 ,-1.393080D+1 , &
     &         -9.715490D+0 ,-7.602160D+0 ,-1.116960D+1 ,-1.263470D+1 , &
     &         -1.413790D+1 ,-1.749010D+1 ,-2.211240D+1 ,-2.025800D+1 , &
     &         -2.491350D+1 ,-2.640370D+1 ,-2.785870D+1 ,-3.016340D+1 , &
     &         -3.251240D+1 ,-3.783360D+1 ,-3.409130D+1 ,-2.897350D+1 , &
     &         -2.838890D+1 ,-2.667700D+1 ,-2.650010D+1 ,-2.489750D+1 , &
     &         -2.566440D+1 ,-2.796680D+1 ,-2.900460D+1 ,-3.095620D+1 , &
     &         -2.552350D+1 ,-2.410880D+1 ,-2.321240D+1 ,-4.353260D+0 , &
     &         -3.653430D+0 ,-2.693910D+0/
!
      IFLAG=1  
      IF (ENER .LT. 50.0D0 .OR. ENER .GT. 30000D0) THEN
        OMEGA=0.0D0
        IFLAG=-4
        RETURN
      ENDIF
!         
      IF (IZ .LT. 3 .OR. IZ .GT. 92) THEN
        OMEGA=0.0D0
        IFLAG=-3
        RETURN
      ENDIF      
!
      IF (IA .LT. 1 .OR. IA .GT. 3) THEN
        OMEGA=0.0D0
        IFLAG=-2
        RETURN
      ENDIF
!
      IF ((IZ .NE. 6) .AND. (IZ .NE. 15) .AND. (IZ .NE. 34) .AND.       &
     &(IA .NE. 1)) THEN
        OMEGA=0.0D0
        IFLAG=-1
        RETURN
      ENDIF      
!   
!     DETERMINATION OF INDEX
!
      DO 10 INDEX=1,78
      IF (IZ .EQ. IZE(INDEX) .AND. IA .EQ. IAE(INDEX)) GO TO 11
  10  CONTINUE
      OMEGA=0.0D0
      IFLAG=0
      RETURN
  11  CONTINUE              
!      
      AA(1)=A1(INDEX)
      AA(2)=A2(INDEX)      
      AA(3)=A3(INDEX)
      AA(4)=A4(INDEX)        
      AA(5)=A5(INDEX)       
      BB(1)=B1(INDEX)
      BB(2)=B2(INDEX)      
      BB(3)=B3(INDEX)
      BB(4)=B4(INDEX)        
      BB(5)=B5(INDEX)          
      X=DLOG(ENER)
      SUM=0.0D0      
      DO 303 J=1,5
      IF (ENER .LT. 300.0D0) SUM=SUM*X+AA(J)
      IF (ENER .GE. 300.0D0) SUM=SUM*X+BB(J)      
 303  CONTINUE
      OMEGA=DEXP(SUM)
!
      RETURN
      END
!
!     ##################################################################
!
      SUBROUTINE IMFPELE(IZ,IA,E,XL,K)
!
!     ******************************************************************
!
!     SUBPROGRAM CALCULATES THE IMFP VALUES FROM THE FIT FOR
!     41 ELEMENTS:    
!
!     Li (Z=3),  Be (Z=4), 
!     Graphite (Z=6), Diamond (Z=6), Glassy carbon (Z=6),
!     Na (Z=11), Mg (Z=12), Al (Z=13), Si (Z=14), K  (Z=19), Sc (Z=21),
!     Ti (Z=22), V  (Z=23), Cr (Z=24), Fe (Z=26), Co (Z=27), Ni (Z=28),
!     Cu (Z=29), Ge (Z=32), Y  (Z=39), Nb (Z=41), Mo (Z=42), Ru (Z=44),
!     Rh (Z=45), Pd (Z=46), Ag (Z=47), In (Z=49), Sn (Z=50), Cs (Z=55),
!     Gd (Z=64), Tb (Z=65), Dy (Z=66), Hf (Z=72), Ta (Z=73), W  (Z=74),
!     Re (Z=75), Os (Z=76), Ir (Z=77), Pt (Z=78), Au (Z=79), Bi (Z=83)
!
!     THE ALGORITHM AND THE PARAMETERS ARE TAKEN FROM:
!     S. Tanuma, C.J. Powell, D.R. Penn, Surface Interface Anal.
!     43 (2011) 689. 
!
!     INPUT PARAMETERS:
!     IZ  - ATOMIC NUMBER.
!     IA  - PARAMETER DEFINING THE ALLOTROPE
!             CARBON (IZ = 6)
!               IA = 1 GRAPHITE
!               IA = 2 DIAMOND
!               IA = 3 GLASSY CARBON
!             OTHER ELEMENTS 
!               IA = 1
!     E   - ELECTRON ENERGY (EV)
!
!     OUTPUT PARAMETERS:
!     XL  - IMFP (ANGSTROM)
!     K   - FLAG:
!           K = -2 ENERGY OUT OF RANGE
!           K = -1 ATOMIC NUMBER OUT OF RANGE
!           K = 0  MISSING DATA FOR THE ATOMIC NUMBER OR
!                  THE NUMBER OF ALLOTROPE.
!           K = 1  SUCCESS.
!
!     ******************************************************************
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION EP(83,3),BETA(83,3),GAMMA(83,3),C(83,3),D(83,3)
      DATA((EP(J,I),J=1,83),I=1,3)                                      &
     &/ -1.0D0,   -1.0D0,   7.99D0,  18.44D0,   -1.0D0,  24.93D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   5.92D0,  10.89D0,       &
     & 15.78D0,  16.59D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  4.28D0,   -1.0D0,  12.86D0,  17.68D0,  22.30D0,  26.14D0,       &
     &  -1.0D0,  30.59D0,  33.58D0,  35.47D0,  35.87D0,   -1.0D0,       &
     &  -1.0D0,  15.59D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0,  11.18D0,   -1.0D0,  19.56D0,  23.09D0,       &
     &  -1.0D0,  28.54D0,  30.00D0,  30.61D0,  29.80D0,   -1.0D0,       &
     & 12.59D0,  14.29D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  3.43D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,  19.77D0,  19.69D0,  20.08D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,  15.73D0,       &
     & 19.53D0,  22.86D0,  25.60D0,  28.08D0,  29.66D0,  30.20D0,       &
     & 29.92D0,   -1.0D0,   -1.0D0,   -1.0D0,  13.94D0,                 &
     &5*-1.0D0,  31.16D0,77*-1.0D0, 5*-1.0D0,  22.30D0,77*-1.0D0/
!
      DATA((BETA(J,I),J=1,83),I=1,3)                                    &
     &/ -1.0D0,   -1.0D0, 0.0653D0, 0.0290D0,   -1.0D0, 0.0183D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0, 0.1258D0, 0.0642D0,       &
     &0.0389D0, 0.0311D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &0.1759D0,   -1.0D0, 0.0603D0, 0.0380D0, 0.0263D0, 0.0204D0,       &
     &  -1.0D0, 0.0151D0, 0.0149D0, 0.0132D0, 0.0117D0,   -1.0D0,       &
     &  -1.0D0, 0.0444D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0, 0.0708D0,   -1.0D0, 0.0324D0, 0.0283D0,       &
     &  -1.0D0, 0.0214D0, 0.0204D0, 0.0188D0, 0.0196D0,   -1.0D0,       &
     &0.0693D0, 0.0505D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &0.3045D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0, 0.0292D0, 0.0337D0, 0.0323D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0, 0.0566D0,       &
     &0.0460D0, 0.0372D0, 0.0321D0, 0.0262D0, 0.0241D0, 0.0223D0,       &
     &0.0213D0,   -1.0D0,   -1.0D0,   -1.0D0, 0.0609D0,                 &
     &5*-1.0D0, 0.0139D0,77*-1.0D0, 5*-1.0D0, 0.0153D0,77*-1.0D0/
!
      DATA((GAMMA(J,I),J=1,83),I=1,3)                                   &
     &/ -1.0D0,   -1.0D0, 0.4340D0, 0.1668D0,   -1.0D0, 0.1333D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0, 0.3030D0, 0.1648D0,       &
     &0.1258D0, 0.1186D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &0.3041D0,   -1.0D0, 0.1587D0, 0.1352D0, 0.0942D0, 0.0860D0,       &
     &  -1.0D0, 0.0715D0, 0.0623D0, 0.0587D0, 0.0535D0,   -1.0D0,       &
     &  -1.0D0, 0.0781D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0, 0.1161D0,   -1.0D0, 0.0811D0, 0.0771D0,       &
     &  -1.0D0, 0.0743D0, 0.0732D0, 0.0787D0, 0.0628D0,   -1.0D0,       &
     &0.0911D0, 0.0752D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &0.2512D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0, 0.0909D0, 0.0979D0, 0.0846D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0, 0.0523D0,       & 
     &0.0488D0, 0.0417D0, 0.0428D0, 0.0395D0, 0.0422D0, 0.0444D0,       &
     &0.0524D0,   -1.0D0,   -1.0D0,   -1.0D0, 0.0700D0,                 &
     &5*-1.0D0, 0.1022D0,77*-1.0D0, 5*-1.0D0, 0.1380D0,77*-1.0D0/
!
      DATA((C(J,I),J=1,83),I=1,3)                                       &
     &/ -1.0D0,   -1.0D0,  3.091D0,  1.591D0,   -1.0D0,  1.365D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,  7.846D0,  3.801D0,       &
     & 1.629D0,  1.104D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       & 
     & 5.119D0,   -1.0D0,  5.393D0,  3.436D0,  2.361D0,  1.492D0,       &
     &  -1.0D0,  0.981D0,  1.186D0,  0.965D0,  0.680D0,   -1.0D0,       &
     &  -1.0D0,  1.592D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0,  3.261D0,   -1.0D0,  2.494D0,  2.000D0,       &
     &  -1.0D0,  1.740D0,  1.760D0,  1.733D0,  1.933D0,   -1.0D0,       &
     & 5.607D0,  4.016D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     & 9.227D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,  1.199D0,  2.236D0,  2.049D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,  3.038D0,       &
     & 2.378D0,  1.543D0,  1.502D0,  1.160D0,  1.132D0,  1.046D0,       &
     & 1.424D0,   -1.0D0,   -1.0D0,   -1.0D0,  3.998D0,                 &
     &5*-1.0D0,  1.271D0,77*-1.0D0, 5*-1.0D0,  1.072D0,77*-1.0D0/
!
      DATA((D(J,I),J=1,83),I=1,3)                                       &
     &/ -1.0D0,   -1.0D0,   81.4D0,   31.2D0,   -1.0D0,   23.6D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,  268.1D0,  126.5D0,       &
     &  50.3D0,   30.3D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &   2.1D0,   -1.0D0,  119.5D0,   88.3D0,   73.5D0,   43.0D0,       &
     &  -1.0D0,   33.7D0,   40.9D0,   34.6D0,   25.8D0,   -1.0D0,       &
     &  -1.0D0,   61.4D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0,   43.1D0,   -1.0D0,   69.8D0,   52.0D0,       &
     &  -1.0D0,   48.4D0,   51.8D0,   50.5D0,   65.0D0,   -1.0D0,       &
     & 191.8D0,  145.9D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     & 247.1D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   26.9D0,   56.6D0,   55.8D0,       &
     &  -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,   -1.0D0,  111.4D0,       &
     &  87.4D0,   55.6D0,   50.6D0,   42.6D0,   38.9D0,   35.4D0,       &
     &  48.1D0,   -1.0D0,   -1.0D0,   -1.0D0,  146.3D0,                 &
     &5*-1.0D0,   24.9D0,77*-1.0D0, 5*-1.0D0,   23.3D0,77*-1.0D0/
!
      IF (IDNINT(E) .LT. 50 .OR. IDNINT(E) .GT. 30000) THEN
        K=-2
        RETURN
      ENDIF
      IF (IZ .LT.3 .OR. IZ .GT. 83) THEN
        K=-1
        RETURN
      ENDIF
      IF (EP(IZ,IA) .LT. 0.0D0) THEN
        K=0
        RETURN
      ENDIF
!
      K=1
      XL=E/(EP(IZ,IA)*EP(IZ,IA)*(BETA(IZ,IA)*DLOG(GAMMA(IZ,IA)*E)-      &
     &(C(IZ,IA)/E)+D(IZ,IA)/(E*E)))
!
      RETURN
      END
!
!     ##################################################################
!
      SUBROUTINE IMFPTPP(IZ,IA,E,XL,K)
!
!     ******************************************************************
!
!     PROCEDURE CALCULATES THE INELASTIC MEAN FREE PATH VALUES FOR
!     ELEMENTS USING THE PREDICTIVE FORMULA TPP-2M
!     THE ALGORITHM AND THE PARAMETERS ARE TAKEN FROM:
!
!     Surface Interface Anal. 21 (1994) 165.
!     THE IMPLEMENTEd NUMBERS OF VALENCE ELECTRONS PER ATOM, Nv, 
!     ARE TAKEN FROM:
!     Surface Interface Anal. 35 (2003) 268.
!
!     INPUT PARAMETERS:
!     IZ     - ATOMIC NUMBER.
!     IA  - PARAMETER DEFINING THE ALLOTROPE
!             CARBON (IZ = 6)
!               IA = 1 GRAPHITE
!               IA = 2 DIAMOND
!               IA = 3 GLASSY CARBON
!             PHOSPHORUS (IZ = 15)
!               IA = 1 WHITE
!               IA = 2 RED
!               IA = 3 BLACK
!             SELENIUM (IZ = 34)
!               IA = 1 ALPHA
!               IA = 2 VITREOUS
!               IA = 3,GRAY
!             TIN (IZ = 50)
!               IA = 1 GRAY
!               IA = 2 WHITE
!             OTHER ELEMENTS 
!               IA = 1
!     E      - ELECTRON ENERGY (EV).
!
!     OUTPUT PARAMETERS:
!     XL - INELASTIC MEAN FREE PATH (ANGSTROMS).
!     K  - FLAG
!          K = -3 ATOMIC MASS IS NOT KNOWN 
!          K = -2 ENERGY OUT OF RANGE
!          K = -1 NUMBER OF VALENCE ELECTRONS NOT AVAILABLE
!          K =  0 DENSITY IS NOT KNOWN
!          K =  1 SUCCESS.
!
!     ******************************************************************
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION NV(96)
      DATA NV/  1,  2,  1,  2,  3,  4,  5,  6,  7,  8,  1,  2,  3,  4,  &
     &  5,  6,  7,  8,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12,  &
     &  3,  4,  5,  6,  7,  8,  1,  2,  3,  4,  5,  6, -1,  8,  9, 10,  &
     & 11, 12,  3,  4,  5,  6,  7,  8,  1,  2,  3,  9,  9,  9, -1,  9,  &
     &  8,  9,  9,  9,  9,  9,  9,  8,  9,  4,  5,  6,  7,  8,  9, 10,  &
     & 11, 12,  3,  4,  5, -1, -1, -1, -1, -1, -1,  4,  3,  3, -1, -1,  &
     & -1, -1/
!
      IF (IDNINT(E) .LT. 50 .OR. IDNINT(E) .GT. 30000) THEN
        K=-2
        RETURN
      ENDIF
!
      IF (NV(IZ) .LT. 0) THEN
        K=-1
        RETURN
      ENDIF
      CNV=NV(IZ)
!
      CALL ATMASS(IZ,ATM,K0)
      IF (K0 .NE. 1) THEN
        K=-3
        RETURN
      ENDIF	  	  	 
      CATMASS=ATM
!
      CALL DENS(IZ,IA,RHO,K0)
      IF (K0 .NE. 1) THEN
        K=0
        RETURN
      ENDIF	  	
!
      EG=0.0D0
      IF (IZ .EQ. 5 .AND. IA .EQ. 1) EG=1.55D0
      IF (IZ .EQ. 6 .AND. IA .EQ. 2) EG=5.4D0
      IF (IZ .EQ. 14 .AND. IA .EQ. 1) EG=1.107D0
      IF (IZ .EQ. 32 .AND. IA .EQ. 1) EG=0.67D0
      IF (IZ .EQ. 34 .AND. IA .EQ. 3) EG=0.67D0
      IF (IZ .EQ. 50 .AND. IA .EQ. 1) EG=0.08D0
      IF (IZ .EQ. 52 .AND. IA .EQ. 1) EG=0.33
!
      K=1
      U=CNV*RHO/CATMASS
      EP=28.8D0*DSQRT(U)
      BETA=-0.10D0+0.944D0/DSQRT(EP*EP+EG*EG)+0.069D0*                  &
     &DEXP(0.1D0*DLOG(RHO))
      GAMMA=0.191D0/DSQRT(RHO)
      C=1.97D0-0.91D0*U
      D=53.4D0-20.8D0*U
      XL=E/(EP*EP*(BETA*DLOG(GAMMA*E)-C/E+D/(E*E)))

      RETURN
      END
!
!     ##################################################################
!
      SUBROUTINE ATMASS(IZ,ATM,K)
!
!     ******************************************************************
!
!     INPUT PARAMETERS
!     IZ  - ATOMIC NUMBER.
!
!     OUTPUT PARAMETERS
!     ATM  - ATOMIC WEIGHT
!     K   - FLAG:
!           K = -1 UNSTABLE NUCLEUS.
!           K =  0 ATOMIC NUMBER OUT OF RANGE.
!           K =  1 SUCCESS.
!
!     ******************************************************************
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)                                
      DIMENSION A(96)                                                                                                      
      DATA A                                                            &
     &/  1.00794D0,  4.002602D0,     6.941D0,   9.012182D0,    10.811D0,&
     &   12.0107D0,  14.00674D0,   15.9994D0, 18.9984032D0,   20.1797D0,&
     & 22.989770D0,   24.3050D0, 26.981538D0,    28.0855D0, 30.973761D0,&
     &    32.066D0,   35.4527D0,    39.948D0,    39.0983D0,    40.078D0,&
     & 44.955910D0,    47.867D0,   50.9415D0,    51.9961D0, 54.938049D0,&
     &    55.845D0, 58.933200D0,   58.6934D0,     63.546D0,     65.39D0,&
     &    69.723D0,     72.61D0,  74.92160D0,      78.96D0,    79.904D0,&
     &     83.80D0,   85.4678D0,     87.62D0,   88.90585D0,    91.224D0,&
     &  92.90638D0,     95.94D0,      -1.0D0,     101.07D0, 102.90550D0,&
     &    106.42D0,  107.8682D0,   112.411D0,    114.818D0,   118.710D0,&
     &   121.760D0,    127.60D0, 126.90447D0,     131.29D0, 132.90545D0,&
     &   137.327D0,  138.9055D0,   140.116D0,  140.90765D0,    144.24D0,&
     &      -1.0D0,    150.36D0,   151.964D0,     157.25D0, 158.92534D0,&
     &    162.50D0, 164.93032D0,    167.26D0,  168.93421D0,    173.04D0,&
     &   174.967D0,    178.49D0,  180.9479D0,     183.84D0,   186.207D0,&
     &    190.23D0,   192.217D0,   195.078D0,  196.96655D0,    200.59D0,&
     &  204.3833D0,     207.2D0, 208.98038D0,       -1.0D0,      -1.0D0,&
     &      -1.0D0,      -1.0D0,      -1.0D0,       -1.0D0,  232.0381D0,&
     & 231.03588D0,  238.0289D0,      -1.0D0,       -1.0D0,      -1.0D0,&
     &      -1.0D0/                                                     
!
      ATM=0.0D0
      IF (IZ .LT. 1 .OR. IZ .GT. 96) THEN
        K=0
        RETURN
      ENDIF
!
      IF (A(IZ) .LT. 0.0D0) THEN
        K=-1
        RETURN
      ENDIF
!
      K=1
      ATM=A(IZ)
!
      RETURN
      END
!
!     ##################################################################
!
      SUBROUTINE DENS(IZ,IA,RO,K)
!
!     ******************************************************************
!
!     INPUT PARAMETERS:
!     IZ  - ATOMIC NUMBER.
!     IA  - PARAMETER DEFINING THE ALLOTROPE
!             CARBON (IZ = 6)
!               IA = 1 GRAPHITE
!               IA = 2 DIAMOND
!               IA = 3 GLASSY CARBON
!             PHOSPHORUS (IZ = 15)
!               IA = 1 WHITE
!               IA = 2 RED
!               IA = 3 BLACK
!             SELENIUM (IZ = 34)
!               IA = 1 ALPHA
!               IA = 2 VITREOUS
!               IA = 3,GRAY
!             TIN (IZ = 50)
!               IA = 1 GRAY
!               IA = 2 WHITE
!             OTHER ELEMENTS 
!               IA = 1
!
!     OUTPUT PARAMETERS:
!     RO  - DENSITY (G/CM**3)
!     K   - FLAG:
!           K = -1 GASEOUS ELEMENT OR UNSTABLE NUCLEUS
!           K =  0 ATOMIC NUMBER OR THE NUMBER OF ALLOTROPE OUT OF RANGE
!           K =  1 SUCCESS
!
!     ******************************************************************
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(92,3)
      DATA((A(I,J),I=1,92),J=1,3)                                       & 
     &/   -1.0D0,    -1.0D0,   0.534D0,   1.85D0,    2.34D0,     2.2D0, &
     &    -1.0D0,    -1.0D0,    -1.0D0,   -1.0D0,    0.97D0,    1.74D0, &
     &    2.70D0,  2.3290D0,   1.823D0,   2.07D0,    -1.0D0,    -1.0D0, &
     &    0.89D0,    1.54D0,    2.99D0,  4.506D0,     6.0D0,    7.15D0, &
     &     7.3D0,    7.87D0,    8.90D0,   8.86D0,    8.96D0,    7.14D0, &
     &    5.91D0,  5.3234D0,    5.74D0,   4.39D0,    -1.0D0,    -1.0D0, &
     &    1.53D0,    2.64D0,    4.47D0,   6.52D0,    8.57D0,    10.2D0, &
     &    -1.0D0,    12.1D0,    12.4D0,   12.0D0,    10.5D0,    8.69D0, &
     &    7.31D0,   5.769D0,    6.68D0,   6.24D0,    -1.0D0,    -1.0D0, &
     &    1.93D0,    3.62D0,    6.15D0,  6.770D0,    6.77D0,    7.01D0, &
     &    7.26D0,    7.52D0,    5.24D0,   7.90D0,    8.23D0,    8.55D0, &
     &    8.80D0,    9.07D0,    9.32D0,   6.90D0,    9.84D0,    13.3D0, &
     &    16.4D0,    19.3D0,    20.8D0,  22.59D0,    22.5D0,    21.5D0, &
     &    19.3D0, 13.5336D0,    11.8D0,   11.3D0,    9.79D0,    9.20D0, &
     &    -1.0D0,    -1.0D0,    -1.0D0,    5.0D0,    -1.0D0,    11.7D0, &
     &    15.4D0,    19.1D0,                                            &
     &  5*-1.0D0,   3.513D0,  8*-1.0D0,   2.16D0, 18*-1.0D0,    4.28D0, &
     & 15*-1.0D0,   7.265D0, 42*-1.0D0,                                 &
     &  5*-1.0D0,     1.8D0,  8*-1.0D0,   2.69D0, 18*-1.0D0,    4.79D0, &
     & 58*-1.0D0/
!
      RO=0.0D0
      IF (IZ .LT. 1 .OR. IZ .GT. 96 .OR. IA .LT. 1 .OR. IA .GT. 3) THEN
         K=0
         RETURN
      ENDIF
      IF (A(IZ,IA) .LT. 0.0D0) K=-1
      IF (A(IZ,IA) .GT. 0.0D0) THEN
        K=1
        RO=A(IZ,IA)
      ENDIF
      RETURN
      END
!
!     ##################################################################
!
      FUNCTION FMOM(X)
!
!     SUBPROGRAM CALCULATING THE INTEGRAND FOR MOMENTS OF THE H FUNCTION
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/AAA/XMU,OMEGA,N
!
      CALL HFUNIV(X,OMEGA,H,INDEX)
	  IF (INDEX .NE. 1) THEN
	    WRITE(*,*)H,INDEX
        WRITE(*,*)'PROBLEMS WITH CALCULATING THE H FUNCTION '
        H=0.0D0
        RETURN
	  ENDIF
      C=1.0D0
      DO 10 I=1,N
      C=C*X
  10  CONTINUE
      FMOM=C*H
      RETURN
      END      
!
!     ##################################################################
!
      FUNCTION FDELTA(X)
!
!     SUBPROGRAM CALCULATING THE INTEGRAND FOR PARAMETERS DELTA
!      
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/AAA/XMU,OMEGA,N
!      
      CALL HFUNIV(X,OMEGA,H,INDEX)
	  IF (INDEX .NE. 1) THEN
	    WRITE(*,*)H,INDEX
        WRITE(*,*)'PROBLEMS WITH CALCULATING THE H FUNCTION '
        H=0.0D0
        RETURN
	  ENDIF      
      C=1.0D0
      DO 10 I=1,N
      C=C*X
  10  CONTINUE      
      B=X+XMU
      FDELTA=C*H/B
      RETURN
      END
!
!     ##################################################################
!
      SUBROUTINE DANALYT(Y0,Y1)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/AAA/XMU,OMEGA,N
      CALL HFUNIV(XMU,OMEGA,H,INDEX)
      Y0=2.0D0*(1.0D0-(1.0D0/H))/(XMU*OMEGA)      
      CONST=DSQRT(1.0D0-OMEGA)
      Y1=(2.0D0/OMEGA)*((1.0D0/H)-CONST)
! 
      RETURN
      END     
!
!     ##################################################################
!
      SUBROUTINE ESCDEPTH(XIMFP,OMEGA,BETA,ALPHA,THETA,PSI,ED,EDMOD)
!
!     PROGRAM CALCULATES THE PHOTOELECTRON ESCAPE DEPTH
!
!     INPUT PARAMETERS
!     XIMFP  - INELASTIC MEAN FREE PATH (ANGSTROMS)
!     OMEGA  - SINGLE SCATTERING ALBEDO
!     BETA   - ASYMMETRY PARAMETER
!     ALPHA  - EMISSION ANGLE (DEGREES)
!     THETA  - INCIDENCE ANGLE OF X-RAYS (DEGREES)
!     PSI    - ANGLE BETWEEEN DIRECTION OF X-RAYS AND THE ANALYZER AXIS
!              (DEGREES)
!
!     OUTPUT PARAMETERS
!     ED    - ESCAPE DEPTH (nm)
!     EDMOD - ESCAPE DEPTH FROM SIMPLIFIED FORMULA (nm)
!            
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/AAA/XMU,OMEGA1,N      
      PARAMETER (PI=3.14159265358979323846264338328D0,CONV=180.0D0/PI)      
      EXTERNAL FMOM,FDELTA
!
      OMEGA1=OMEGA
      XMU=DCOS(ALPHA/CONV)      
      CPSI=DCOS(PSI/CONV)
      CTHETA=DCOS(THETA/CONV)      
      CONST=1.0D0/DSQRT(1.0D0-OMEGA)
!      
      E0=1.0D-13
      A=0.0D0
      B=1.0D0          
	  N=1
      CALL GL64(FMOM,A,B,E0,ALPHA1,R,I1)
      N=3
      CALL GL64(FMOM,A,B,E0,ALPHA3,R,I3)      
	  N=1
      CALL GL64(FDELTA,A,B,E0,DELTA1,R,J1)
      N=3
      CALL GL64(FDELTA,A,B,E0,DELTA3,R,J3)            
      CALL HFUNIV(XMU,OMEGA,H,INDEX)      
      ETA1=(OMEGA*BETA/16.0D0)*(3.0D0*CTHETA*CTHETA-1.0D0)*             &
     &(3.0D0*ALPHA3-ALPHA1)      
      ETA2=(OMEGA*BETA/16.0D0)*(3.0D0*CTHETA*CTHETA-1.0D0)*             &
     &(3.0D0*DELTA3-DELTA1)
      XI1=(OMEGA/2.0D0)/(1.0D0-OMEGA)*ALPHA1-ETA1
      XI2=CONST-(BETA/4.0D0)*(3.0D0*CPSI*CPSI-1)/H+ETA2
      XIMOD1=(OMEGA/2.0D0)/(1.0D0-OMEGA)*ALPHA1
      XIMOD2=CONST-(BETA/4.0D0)*(3.0D0*CPSI*CPSI-1.0D0)/H
!
      ED=XIMFP*(1.0D0-OMEGA)*(XMU+XI1/XI2)
      EDMOD=XIMFP*(1.0D0-OMEGA)*(XMU+XIMOD1/XIMOD2)
!
      RETURN
      END 
!
!     ##################################################################
!
      SUBROUTINE SIGINT(XIMFP,OMEGA,BETA,ALPHA,THETA,PSI,SIG,SIGMOD)
!
!     PROGRAM CALCULATES THE PHOTOELECTRON SIGNAL INTENSITY
!
!     INPUT PARAMETERS
!     XIMFP  - INELASTIC MEAN FREE PATH (ANGSTROMS)
!     OMEGA  - SINGLE SCATTERING ALBEDO
!     BETA   - ASYMMETRY PARAMETER
!     ALPHA  - EMISSION ANGLE (DEGREES)
!     THETA  - INCIDENCE ANGLE OF X-RAYS (DEGREES)
!     PSI    - ANGLE BETWEEEN DIRECTION OF X-RAYS AND THE ANALYZER AXIS
!              (DEGREES)
!
!     OUTPUT PARAMETERS
!     SIG    - SIGNAL INTENSITY (ARB. UNITS)
!     SIGMOD - SIGNAL INTENSITY FROM SIMPLIFIED FORMULA (ARB. UNITS)
!
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/AAA/XMU,OMEGA1,N      
      PARAMETER (PI=3.14159265358979323846264338328D0,CONV=180.0D0/PI)
      EXTERNAL FDELTA
!
      OMEGA1=OMEGA
      XMU=DCOS(ALPHA/CONV)      
      CPSI=DCOS(PSI/CONV)
      CTHETA=DCOS(THETA/CONV)      
      CONST=1.0D0/DSQRT(1.0D0-OMEGA)      
!
      E0=1.0D-13
      A=0.0D0
      B=1.0D0          
	  N=1
      CALL GL64(FDELTA,A,B,E0,DELTA1,R1,I1)
      N=3
      CALL GL64(FDELTA,A,B,E0,DELTA3,R3,I3)        
      CALL HFUNIV(XMU,OMEGA,H,INDEX)
      ETA=(OMEGA*BETA/16.0D0)*(3.0D0*CTHETA*CTHETA-1.0D0)*H*            &
     &(3.0D0*DELTA3-DELTA1)
      XI=CONST*H-(BETA/4.0D0)*(3.0D0*CPSI*CPSI-1.0D0)
      SIG=XIMFP*(1.0D0-OMEGA)*(XI+ETA)
      SIGMOD=XIMFP*(1.0D0-OMEGA)*XI     
!
      RETURN
      END                   
!
!     ##################################################################
!
      SUBROUTINE GL64(FF,A,B,E0,V,R,I1)
!
!     SUBPROGRAM CALCULATES THE INTEGRAL OF A GIVEN FUNCTION OVER
!     DEFINED INTERVAL USING THE GAUSS-LEGENDRE QUADRATURE (N = 64)
!
!     INPUT PARAMETERS
!     FF - FUNCTION OF THE FORM FF(X) PROVIDING THE VALUE OF THE
!          INTEGRAND FOR ARGUMENT X.
!     A  - LOWER LIMIT OF INTEGRATION.
!     B  - UPPER LIMIT OF INTEGRATION.
!     E0 - DESIRED RELATIVE ACCURACY.
!
!     OUTPUT PARAMETERS
!     V  - VALUE OF THE INTEGRAL.
!     R  - REACHED RELATIVE ACCURACY.
!     I1 - FLAG
!          I1 =  0 ASSUMED ACCURACY CANNOT BE REACHED.
!          I1 =  1 SUCCESS.
!    
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION X64(32),W64(32)
      DIMENSION X(128),W(128)
!
!     ABSCISSAS
      DATA X64/0.024350292663424432509D0,0.072993121787799039450D0,     &
     &         0.121462819296120554470D0,0.169644420423992818037D0,     &
     &         0.217423643740007084150D0,0.264687162208767416374D0,     &
     &         0.311322871990210956158D0,0.357220158337668115950D0,     &
     &         0.402270157963991603696D0,0.446366017253464087985D0,     &
     &         0.489403145707052957479D0,0.531279464019894545658D0,     &
     &         0.571895646202634034284D0,0.611155355172393250249D0,     &
     &         0.648965471254657339858D0,0.685236313054233242564D0,     &
     &         0.719881850171610826849D0,0.752819907260531896612D0,     &
     &         0.783972358943341407610D0,0.813265315122797559742D0,     &
     &         0.840629296252580362752D0,0.865999398154092819761D0,     &
     &         0.889315445995114105853D0,0.910522137078502805756D0,     &
     &         0.929569172131939575821D0,0.946411374858402816062D0,     &
     &         0.961008799652053718919D0,0.973326827789910963742D0,     &
     &         0.983336253884625956931D0,0.991013371476744320739D0,     &
     &         0.996340116771955279347D0,0.999305041735772139457D0/
!     WEIGHTS
      DATA W64/0.048690957009139720383D0,0.048575467441503426935D0,     &
     &         0.048344762234802957170D0,0.047999388596458307728D0,     &
     &         0.047540165714830308662D0,0.046968182816210017325D0,     &
     &         0.046284796581314417296D0,0.045491627927418144480D0,     &
     &         0.044590558163756563060D0,0.043583724529323453377D0,     &
     &         0.042473515123653589007D0,0.041262563242623528610D0,     &
     &         0.039953741132720341387D0,0.038550153178615629129D0,     &
     &         0.037055128540240046040D0,0.035472213256882383811D0,     &
     &         0.033805161837141609392D0,0.032057928354851553585D0,     &
     &         0.030234657072402478868D0,0.028339672614259483228D0,     &
     &         0.026377469715054658672D0,0.024352702568710873338D0,     &
     &         0.022270173808383254159D0,0.020134823153530209372D0,     &
     &         0.017951715775697343085D0,0.015726030476024719322D0,     &
     &         0.013463047896718642598D0,0.011168139460131128819D0,     &
     &         0.008846759826363947723D0,0.006504457968978362856D0,     &
     &         0.004147033260562467635D0,0.001783280721696432947D0/
!
      I1=1
      NPOINT=64
      M=(NPOINT+1)/2
      DO 64 I=1,M
      X(I)=-X64(M-I+1)
      W(I)=W64(M-I+1)
  64  CONTINUE
      DO 65 I=1,M
      X(I+M)=X64(I)
      W(I+M)=W64(I)
  65  CONTINUE
!
      MINT=1
      SUM=0.0D0
      C1=(B-A)/2.0D0
      C2=(B+A)/2.0D0
      DO 100 J=1,NPOINT
      ZI=C1*X(J)+C2
      SUM=SUM+W(J)*FF(ZI)
 100  CONTINUE
      VPREV=C1*SUM
!
      K=1
 200  CONTINUE
      V=0.0D0
      MINT=2*MINT
      H=(B-A)/DBLE(MINT)
      DO 201 I=1,MINT
      SUM=0.0D0
      C1=H/2.0D0
      C2=A+H*DBLE(I)-C1
      DO 202 J=1,NPOINT
      ZI=C1*X(J)+C2
      SUM=SUM+W(J)*FF(ZI)
 202  CONTINUE
      V=V+C1*SUM
 201  CONTINUE
      R=DABS((VPREV-V)/V)
      IF (K .GT. 15) THEN
        I1=0
        RETURN
      ENDIF
      VPREV=V
      K=K+1
      IF (R .GT. E0) GOTO 200
!
      RETURN
      END
